package com.sagacloud.route.processors.Maintainance;
/*
 * Author: Jxing
 * Create Time: 2018/7/13
 */

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.Exceptions.InvalidPostException;
import com.sagacloud.pojos.DPSelectPropertyResult;
import com.sagacloud.pojos.InsurancePost;
import com.sagacloud.route.processors.Insurance.GetPropertyUnderWarrantyProcessor;
import com.sagacloud.utils.Const;
import com.sagacloud.utils.VendersUtil;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;

import java.util.*;

public class FilterPropertyProcessor implements Processor {
    private static Set<String> returnInfoNames = VendersUtil.createSet("EquipLocalName", "EquipLocalId", "Brand",
            "Product", "Specification", "MaintainDeadline", "MaintainPeriod");
    @Override
    public void process(Exchange exchange) throws Exception {
        String jsonStr = exchange.getIn().getBody(String.class);
        DPSelectPropertyResult dpResult = JSONObject.parseObject(jsonStr, DPSelectPropertyResult.class);
//        System.out.println(jsonStr);
        if(dpResult.getResult() == null && !dpResult.getResult().equals("success"))
            throw new InvalidPostException("数据平台返回异常!");

        InsurancePost post = (InsurancePost) exchange.getProperty("postParam");
        List<Map<String, Object>> propertyList = dpResult.getContent();

        if(propertyList.size() == 0)
        {
            exchange.getOut().setBody(VendersUtil.successJsonMsg("", new JSONArray()));
            return;
        }
        Set<String> tmp = null;
        for(int i = propertyList.size() - 1; i > -1; --i){
            Map<String, Object> property = propertyList.get(i);
            Map<String, Object> infos = (Map<String, Object>) property.get("infos");
            // 过滤掉不是此保险商保的资产
            if(infos.get(Const.MTN_ID) == null || !post.getVenderId().equals(infos.get(Const.MTN_ID).toString())){
                propertyList.remove(i);
            }else{
                // 将报废日期过期的/报废日期未到的资产过滤掉(根据post的expire字段决定)
                // 暂时将没有报废日期信息点的资产当作没有作废的资产
                boolean isPropertyExpire = false;
                if(infos.get(Const.MTN_DEADLINE) != null){
                    String deadline = infos.get(Const.MTN_DEADLINE).toString();
                    try {
                        Date expireDate = GetPropertyUnderWarrantyProcessor.sdf.parse(deadline);
                        Date now = new Date();
                        if(now.compareTo(expireDate) >= 0)
                            isPropertyExpire = true;
                    }catch (Exception ignore)
                    {}
                }
                // 过滤掉 (expire = true, isPropertyExpire = false) (expire = false, isPropertyExpire = true)的两种情况下资产
                if((post.isExpire() && !isPropertyExpire)||(!post.isExpire() && isPropertyExpire)) {
                    propertyList.remove(i);
                    continue;
                }
                tmp = new HashSet<>();
                for(String infoName : infos.keySet()){
                    if(!returnInfoNames.contains(infoName) && !tmp.contains(infoName))
                        tmp.add(infoName);
                }
                for(String infoName : tmp)
                    infos.remove(infoName);
            }
        }
        exchange.getOut().setBody(VendersUtil.successJsonMsg("", (JSONArray) JSONArray.toJSON(propertyList)));
    }
}
